/* BeaSTM (c) Thorbjörn Jemander */

#ifndef STM_RANGE_SLIDER_H
#define STM_RANGE_SLIDER_H

#define STM_ARROW_UP		0
#define STM_ARROW_RIGHT		1
#define STM_ARROW_DOWN		2
#define STM_ARROW_LEFT		3

class JRangeSlider;

#include <ScrollBar.h>
#include <View.h>
#include <interface/GraphicsDefs.h>

#define min(x,y) (x>y?y:x)
#define max(x,y) (x>y?x:y)
#define ONE_UNIT BPoint(1,1)

class JRangeSlider : public BView
{
	virtual
	void 		Draw( BRect argUpdateRect );
	virtual
	void		MouseDown( BPoint argDownPoint );

	void		CalculateNewRectangles();
	float		HeightOfSlider();
	bool		BoundUpperAndLowerPositions( 	float argUpperPosition, float argLowerPosition );

	void		IncUppAndLowWhileLMBPressed(	uint32	argButtons, BPoint argDownPoint, float argStep, bool argRepetitive = true);		// LMB = Left Mouse Button (Primary Mouse Button)
	void		DecUppAndLowWhileLMBPressed(	uint32	argButtons, BPoint argDownPoint, float argStep, bool argRepetitive = true);		// RMB	= Right Mouse Button (Secondary Mouse Button)
	void		IncUppWhileRMBPressed(			uint32	argButtons, BPoint argDownPoint, float argStep, bool argRepetitive = true);		// AMB	= Any Mouse Button
	void		DecUppWhileRMBPressed(			uint32	argButtons, BPoint argDownPoint, float argStep, bool argRepetitive = true);
	void		IncLowWhileRMBPressed(			uint32	argButtons, BPoint argDownPoint, float argStep, bool argRepetitive = true);
	void		DecLowWhileRMBPressed(			uint32	argButtons, BPoint argDownPoint, float argStep, bool argRepetitive = true);
	void		UppAndLowFollowMouseWhileAMBPressed(uint32	argButtons, BPoint argDownPoint);
	void		UppFollowMouseWhileAMBPressed(	uint32	argButtons, BPoint argDownPoint);
	void		LowFollowMouseWhileAMBPressed(	uint32	argButtons, BPoint argDownPoint);

	float		// Value measures:
				MinValue,		// Minimum value of slider corr. to bottom.
				MaxValue,		// Maximum value of slider corr. to top.
				UpperPosition,	// Value between MinValue and MaxValue, corr to top of slider.
				LowerPosition,	// Value between MinValue and MaxValue, corr to bottom of slider.
				ArrowStep,		// Step the slider takes when pressing an arrow button, in terms of value!
				// Pixel measures:
				Width,			// Width of view in pixels.
				Height,			// Height of view in pixels.
				DisplayRange,	// The height of the container in pixels.
				KnobSize,		// In pixels.
				// Conversion factors:
				ValueToPixelFactor,	// Mapping between the position value and the pixel value. 				
				PixelToValueFactor; // Mapping between the pixel value and the position value.
				
	orientation	Direction;

	BRect		ArrowUpAboveSlider,
				ArrowDownAboveSlider,
				ContainerAboveSlider,
				SliderTopSizeKnob,
				SliderMoveKnob,
				SliderBottomSizeKnob,
				ContainerBelowSlider,
				ArrowUpBelowSlider,
				ArrowDownBelowSlider;

	bool		ArrowUpAboveSliderPressed,
				ArrowDownAboveSliderPressed,
				ContainerAboveSliderPressed,
				SliderTopSizeKnobPressed,
				SliderMoveKnobPressed,
				SliderBottomSizeKnobPressed,
				ContainerBelowSliderPressed,
				ArrowUpBelowSliderPressed,
				ArrowDownBelowSliderPressed;

	int32		TimeBetweenUpdates;

public:

	rgb_color	DarkGrey,
				MediumDarkGrey,	
				MediumLightGrey,
				LightGrey,
				White;

	bool	SetUpperPosition( 		float argValue );	
	bool	SetLowerPosition( 		float argValue );	
	void	SetTimeBetweenUpdates(	int32 argDelayTime );
	void	SetArrowStep( float argValue )		{ ArrowStep = argValue; }

			JRangeSlider(	BRect		argFrame,
							const char	*argName,
							float		argMin,
							float		argMax,
							float		argLowerPosition,
							float		argUpperPosition,
							orientation	argDirection,
							float		argArrowStep = 5);

	virtual
	void		DrawInRect(	BRect argRect, bool	argActivated);
	virtual
	void		DrawOutRect(BRect argRect, bool argActivated);
	virtual
	void		DrawArrowIn(BRect argRect, int32 argDirection, bool argActivated); 
	virtual
	void		UpperPositionChanged(	float argNewUpperPosition );
	virtual	
	void		LowerPositionChanged( 	float argNewLowerPosition );
	virtual
	void		BothUpperAndLowerPositionsChanged( 	float argNewUpperPosition, 	float argNewLowerPosition  );
};

#endif